<?php
namespace Teknisa\EST\Modules\Lancamentos\Controllers;

use Teknisa\EST\Services\ValidacoesEstoqueService;
use \Teknisa\EST\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO;

class Est58000ItemRetirada extends TeknisaCrud {

    protected $dataSourceName = "itlanctoest_retirada";
    protected $dataSourceNameToSave = "itlanctoest";
    protected $parameterBagColumns = ['CDFILIMOVI','NRLANCESTQ'];

    protected $Est58000Service;
    protected $validacoesEstoqueService;

    protected $connection;

    public function __construct($crudService, $parameterBag, $Est58000Service, ValidacoesEstoqueService $validacoesEstoqueService, $entityManager) {
        parent::__construct($crudService,$parameterBag);

        $this->Est58000Service = $Est58000Service;
        $this->validacoesEstoqueService = $validacoesEstoqueService;

        $this->connection = $entityManager->getConnection();
    }

    public function save(DTO\Request $request, DTO\Response $response) {
        $itens = $request->getDataSet()->getRows();
        $this->connection->beginTransaction();
        try {
            $this->Est58000Service->verificaInventarioRotativo($itens[0]['CDFILIMOVI'],
                                                              substr($itens[0]['DTLANCMOVI'], 0, 10));

            if($itens[0]['__is_new']) {
                $lancamento = $this->Est58000Service->insertItensForRetirada($itens, substr($itens[0]['DTLANCMOVI'], 0, 10),
                                                               $itens[0]['CDTIPORETI'],
                                                               $itens[0]['CDCENTCUST'],
                                                               $itens[0]['NRREQUESTO']);
                                                                
            } else {
                $lancamento = $this->Est58000Service->editItemOfRetirada($itens[0]);
            }

            $this->validacoesEstoqueService->atualizaEstoque($lancamento["CDFILIAL"], $lancamento["NRLANCESTQ"]);

            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $item = $request->getRow();
        $this->connection->beginTransaction();
        try {
            $this->Est58000Service->verificaInventarioRotativo($item['CDFILIMOVI'],
                                                              substr($item['DTLANCMOVI'], 0, 10));

            $this->Est58000Service->processDeleteItem($item);

            $this->validacoesEstoqueService->atualizaEstoque($item["CDFILIAL"], $item["NRLANCESTQ"]);

            $this->connection->commit();
        } catch(\Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}